@@ -24,6 +24,7 @@
 // Copyright (C) 2020 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2020 Marek Kasik <mkasik@redhat.com>
 // Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -569,7 +570,7 @@ class LinkUnknown : public LinkAction
 {
 public:
     // Build a LinkUnknown with the specified action type.
-    explicit LinkUnknown(const char *actionA);
+    explicit LinkUnknown(std::string &&actionA);
 
     ~LinkUnknown() override;
 
@@ -601,10 +602,10 @@ public:
     Links(const Links &) = delete;
     Links &operator=(const Links &) = delete;
 
-    const std::vector<AnnotLink *> &getLinks() const { return links; }
+    const std::vector<std::shared_ptr<AnnotLink>> &getLinks() const { return links; }
 
 private:
-    std::vector<AnnotLink *> links;
+    std::vector<std::shared_ptr<AnnotLink>> links;
 };
 
 #endif
